classdef DataManager <  handle
    % objekt na spracovanie MNIST datasetu
    
    properties
        trainData;
        trainLabels;
        valData;
        valLabels;
        curbatch;
        mbSize;
    end
    
    methods
        %constructor
        %path - mnist.mat
        %valsize - desired size of validation set
        
        function obj = DataManager(path,valSize)
            MNIST = load(path);            
            obj.trainData = MNIST.training.images(:,:,1:end-valSize);
            obj.valData = MNIST.training.images(:,:,end-valSize+1:end);
            L = MNIST.training.labels();
            nL = [L == 0, L == 1, L == 2, L == 3, L == 4, L == 5, L == 6, L == 7, L == 8, L == 9];
            obj.trainLabels = nL(1:end-valSize,:);
            obj.valLabels = L(end-valSize+1:end);
            obj.curbatch = 0;
            obj.mbSize = 1;
        end
        
        %returns size of the validation set
        function s = valSize(obj)
            s = size(obj.valData,3);
        end
        
        %returns size of the training set
        function s = trainSize(obj)
            s = size(obj.trainData,3);
        end
        
        %return matrix with each column representing
        %one training example as a vector
        function vecIm = valImVecs(obj)
            vecIm = reshape(obj.valData,28*28,[]);
        end
        
        %return i-th validation image
        function im = valIm(obj,i)
            im = obj.valData(:,:,i);
        end
        
        %return i-th training image in vector form
        function vecIm = trainImVec(obj,i)
            vecIm = reshape(obj.trainData(:,:,i),[],1);
        end
        
        %return i-th training image
        function im = trainIm(obj,i)
            im = obj.trainData(:,:,i);
        end
        
        %return label vector for i-th training example
        %with all zeros, except on the index with
        %correct label
        function l = trainLabVec(obj,i)
            l = obj.trainLabels(i,:)';
        end
        
        %returns array of correct labels from
        %the validation set
        function l = valLab(obj)
            l = obj.valLabels';
        end
        
        %reshuffle the training set
        %prepare counters for minibatch fetching
        function obj = initBatches(obj,mbSize)
            p = randperm(obj.trainSize);
            obj.trainData = obj.trainData(:,:,p);
            obj.trainLabels = obj.trainLabels(p,:);
            obj.curbatch = 0;            
            obj.mbSize = mbSize;
        end
        
        %returns arrays with training data and labels
        %for the next minibatch
        function [imVec,labVec] = getBatch(obj)
            nextpos = min(obj.curbatch + obj.mbSize, obj.trainSize);
            imVec = reshape(obj.trainData(:,:,obj.curbatch+1:nextpos),28*28,[]);
            labVec = obj.trainLabVec(obj.curbatch+1:nextpos);
            obj.curbatch = nextpos;
        end
        
        %returns wheter further minibatch can be fetched       
        function c = cont(obj)
            if obj.curbatch == obj.trainSize
                c = false;
            else
                c = true;
            end
        end            
    end
    
    
end
    